/*
 * Copyright (c) 2007-2009 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <string.h>

#include "mfs_config.h"
#include "lwip/inet.h"
#include "lwip/sockets.h"


#include "lwip/opt.h"

#include "lwip/debug.h"
#include "lwip/def.h"
#include "lwip/sys.h"
#include "lwip/mem.h"
#include "lwip/stats.h"

#include "webserver.h"
#include "platform_fs.h"
#include "xparameters.h"

#include "xgpio.h"
#include "xil_cache.h"
#include "core_portme.h"

XGpio GpioInput;  /* The driver instance for GPIO Device configured as I/P */

/* following constant is used to determine which channel of the GPIO is
 * used if there are 2 channels supported in the GPIO.
 */
#define LED_CHANNEL 1
#define BUFSIZE 1024
#define LED_MAX_BLINK	0x1	/* Number of times the LED Blinks */

#define GPIO_BITWIDTH	16	/* This is the width of the GPIO */
#define LEDS_TOGGLE XPAR_GPIO_0_BASEADDR
char *notfound_header =
	"<html> \
	<head> \
		<title>404</title> \
  		<style type=\"text/css\"> \
		div#request {background: #eeeeee} \
		</style> \
	</head> \
	<body> \
	<h1>404 Page Not Found</h1> \
	<div id=\"request\">";

char *notfound_footer =
	"</div> \
	</body> \
	</html>";
char *js_header = "<html> \
	<head> \
		<title>LEDs</title> \
  		<style type=\"text/css\"> \
		div#request {background: #eeeeee} \
		</style> \
	</head> \
	<body> \
	<h1>LEDs Value</h1> \
	<div id=\"request\"> \
	<script type=\"text/javascript\"> \
	var switchval; \
	switchval = \"";

char *js_footer = "\" \
	document.write(switchval); \
	</script> \
	</div> \
	</body> \
	</html>";
/* dynamically generate 404 response:
 *	this inserts the original request string in betwween the notfound_header & footer strings
 */
#pragma GCC optimize 0
volatile int taskrun=0;
volatile int ledon=0;
void * xToggletask;
char str2[20]; //  for push button status update
//Total tick from coremark results.
extern CORE_TICKS total_time;
char num1[20]="\0",num2[20]="\0";
float f1,f2;
float stof(const char* s);
void toggle_task();
#define UNUSED(buf1) (void)(buf1)
int is_cmd_ledon(char *buf)  { return (!strncmp(buf, "cmd", 3) && !strncmp(buf + 4, "ledson",   6)); }
int is_cmd_ledpattern(char *buf)  { return (!strncmp(buf, "cmd", 3) && !strncmp(buf + 4, "ledspat",   7)); }
int is_cmd_ledsoff(char *buf) { return (!strncmp(buf, "cmd", 3) && !strncmp(buf + 4, "ledsoff",  7)); }
int is_cmd_fpuadd(char *buf) { return (!strncmp(buf, "cmd", 3) && !strncmp(buf + 4, "fpuadd",  6)); }
int is_cmd_fpusub(char *buf) { return (!strncmp(buf, "cmd", 3) && !strncmp(buf + 4, "fpusub",  6)); }
int is_cmd_fpumul(char *buf) { return (!strncmp(buf, "cmd", 3) && !strncmp(buf + 4, "fpumul",  6)); }
int is_cmd_pushb(char *buf) { return (!strncmp(buf, "cmd", 3) && !strncmp(buf + 4, "pushb",  5)); }
int is_cmd_cmrk(char *buf) { return (!strncmp(buf, "cmd", 3) && !strncmp(buf + 4, "coremark",  8)); }

int do_404(int sd, char *req, int rlen)
{
	int len, hlen;
	char buf[BUFSIZE];

	len = strlen(notfound_header) + strlen(notfound_footer) + rlen;

	hlen = generate_http_header(buf, "html", len);
	if (lwip_write(sd, buf, hlen) != hlen) {
		xil_printf("error writing http header to socket\r\n");
		xil_printf("http header = %s\n\r", buf);
		return -1;
	}

	lwip_write(sd, notfound_header, strlen(notfound_header));
	lwip_write(sd, req, rlen);
	lwip_write(sd, notfound_footer, strlen(notfound_footer));

	return 0;
}
float stof(const char* s){
  float rez = 0, fact = 1;
  if (*s == '-'){
    s++;
    fact = -1;
  };
  int point_seen=0;

  for (point_seen = 0; *s; s++){
    if (*s == '.'){
      point_seen = 1;
      continue;
    };
    int d = *s - '0';
    if (d >= 0 && d <= 9){
      if (point_seen) fact /= 10.0f;
      rez = rez * 10.0f + (float)d;
    };
  };
  return rez * fact;
}


// reverses a string 'str' of length 'len'
 void reverse(char *str, int len)
{
    int i=0, j=len-1, temp;
    while (i<j)
    {
        temp = str[i];
        str[i] = str[j];
        str[j] = temp;
        i++; j--;
    }
}

 // Converts a given integer x to string str[].  d is the number
 // of digits required in output. If d is more than the number
 // of digits in x, then 0s are added at the beginning.
int intToStr(int x, char str[], int d)
{
    int i = 0;
    while (x)
    {
        str[i++] = (x%10) + '0';
        x = x/10;
    }

    // If number of digits required is more, then
    // add 0s at the beginning
    while (i < d)
        str[i++] = '0';

    reverse(str, i);
    str[i] = '\0';
    return i;
}

// Converts a floating point number to string.
void ftoa(float n, char *res, int afterpoint)
{
    // Extract integer part
    int ipart = (int)n;

    // Extract floating part
    float fpart = n - (float)ipart;

    // convert integer part to string
    int i = intToStr(ipart, res, 0);

    // check for display option after point
    if (afterpoint != 0)
    {
        res[i] = '.';  // add dot

        // Get the value of fraction part upto given no.
        // of points after dot. The third parameter is needed
        // to handle cases like 233.007
        fpart = fpart * pow(10, afterpoint);

        intToStr((int)fpart, res + i + 1, afterpoint);
    }
}
unsigned volatile char * convert_fnum(char *str)
{
	int i,j,cx=0,cnt=0;
	for(i=0;str[i];i++)
		{
			if(str[i]=='/')
			{
				for(j=i+1;str[j];j++)
				{
					num2[cx]=str[j];
					cx++;
					if(str[j]==' ')
					{
					cx=1;
					break;
					}
				}
			cnt++;
			}

				if(cnt>=1)
				{
					break;
				}
	num1[i]=str[i];
		}
f1=stof(num1);
f2=stof(num2);
}
void push_button_thread(void *parameter)
{
	unsigned int Button;
//	const unsigned char ucSetToOutput = 0U;
	int Status;

	 /*
	  * Initialize the GPIO driver so that it's ready to use,
	  * specify the device ID that is generated in xparameters.h
	  */
	 Status = XGpio_Initialize(&GpioInput, XPAR_AXI_GPIO_1_DEVICE_ID);
	 if (Status != XST_SUCCESS) {
		  return XST_FAILURE;
	 }

	 /*
	  * Set the direction for all signals to be inputs
	  */
	 XGpio_SetDataDirection(&GpioInput, LED_CHANNEL, 0xFFFFFFFF);

	 /*
	  * Read the state of the data so that it can be  verified
	  */
	while(1)
	{
		Button = XGpio_DiscreteRead(&GpioInput, LED_CHANNEL);

		if(Button)
		{
		        	        	if(Button==1)
		        	        	{

		        	        		strcpy(str2,"SW1 Pressed");
		        	        		xil_printf("SW1 Pressed.....\n");
		        	        	}
		        	        	if(Button==2)
		        	        	{
		        	        		strcpy(str2,"SW2 Pressed");
		        	        		xil_printf("SW2 Pressed.....\n");
		        	        	}
		        	        	if(Button==4)
		        	        	{
		        	        		strcpy(str2,"SW3 Pressed");
		        	        		xil_printf("SW3 Pressed.....\n");
		        	        	}
		        	        	if(Button==8)
		        	        	{
		        	        		strcpy(str2,"SW4 Pressed");
		        	        		xil_printf("SW4 Pressed.....\n");
		        	        	}
		}

		vTaskDelay(100);

	}
}
int  main_start(void);
int do_http_post(int sd, char *req, int rlen)
{
	xil_printf("INSIDE COMMAND PROCESSING----- %s \n",req);

	char buf[BUFSIZE];
	int len = 0;
   static char res[20];
    	if(is_cmd_ledon(req+6))
        {
        	static char tbuf[15];
        	strcpy(tbuf,"ledson");
        	taskrun=1;
        	ledon=1;
        	*(volatile unsigned int*)LEDS_TOGGLE=0xFF;
         	generate_http_header(buf, "txt", 1);
          	lwip_write(sd, buf, len);
          	lwip_write(sd,tbuf, strlen(tbuf));

        }
        if(is_cmd_ledsoff(req+6))
        {
        	static char tbuf[15];
        	strcpy(tbuf,"ledsoff");
        	taskrun=1;ledon=0;
        	*(volatile unsigned int*)LEDS_TOGGLE=0x0;
         	generate_http_header(buf, "txt", 1);
          	lwip_write(sd, buf, len);
          	lwip_write(sd,tbuf, strlen(tbuf));

        }
        if(is_cmd_ledpattern(req+6))
        {
        	static char tbuf[15];
        	strcpy(tbuf,"ledspat");
        	taskrun=0;ledon=0;
        	xTaskCreate( toggle_task, "toggle_task", 100, NULL, 2, &xToggletask);
         	generate_http_header(buf, "txt", 1);
            lwip_write(sd, buf, len);
         	lwip_write(sd,tbuf, strlen(tbuf));
        }
        if(is_cmd_fpuadd(req+6))
        {
        	//Here Unused buf1 used for memory alignment issue..need to fix
        	char buf1[1400] __attribute__ ((unused)) __attribute__((optimize("O0")));
        	char str[100];
        	strcpy(str,req+17);
        	convert_fnum(str);

        	float add=f1+f2;
        	ftoa(add, res, 6);

        	xil_printf("Length : %d\n",strlen(res));
        	xil_printf("STRING %s\n",res);

         	generate_http_header(buf, "txt", 1);
          	lwip_write(sd, buf, len);
          	lwip_write(sd,res, strlen(res));
          	strncpy(num1,"\0",20);
          	strncpy(num2,"\0",20);
          	strncpy(res,"\0",20);

        }
        if(is_cmd_fpusub(req+6))
        {
         	        	int min=0;;
        	        	char tbuf[15],str[100];
        	        	strcpy(str,req+17);
        	        	convert_fnum(str);
        	        	float sub=f1-f2;
        	        	if(sub<0)
        	        	{
        	        	   sub=-sub;
        	        	   min++;
        	        	}

        	        	ftoa(sub, res, 6);
        	        	generate_http_header(buf, "txt", 1);
        	        	lwip_write(sd, buf, len);
        	        	if(min==1)
        	        	{
        	        		strcpy(tbuf,"-");
        	        		strcat(tbuf,res);
        	        		min=0;
        	        		memset(res, 0, 20);
        	        		strcpy(res,tbuf);
        	        	}
        	        	lwip_write(sd,res, strlen(res));
        	          	strncpy(num1,"\0",20);
        	          	strncpy(num2,"\0",20);
        	          	strncpy(res,"\0",20);
        }
        if(is_cmd_fpumul(req+6))
        {
        				char str[100];
          	        	strcpy(str,req+17);
        	        	convert_fnum(str);
         	        	float mul=f1*f2;
        	        	ftoa(mul, res, 6);
        	        	generate_http_header(buf, "txt", 1);
        	          	lwip_write(sd, buf, len);
        	          	lwip_write(sd,res, strlen(res));
        	          	strncpy(num1,"\0",20);
        	          	strncpy(num2,"\0",20);
        	          	strncpy(res,"\0",20);
        }
        if(is_cmd_cmrk(req+6))
        {
        	int x;
        	char str[100];
        	x=main_start();
        	static char iter[10],ttick[15];
        	intToStr(3000,iter,4);
        	intToStr(total_time,ttick,8);
        	strncpy(str,"\0",100);
        	strcpy(str,"/");
        	strcat(str,"2000");
        	strcat(str,"/");
        	strcat(str,ttick);
        	strcat(str,"/");
        	xil_printf("The sendng str: %s\n",str);

        	generate_http_header(buf, "txt", 1);
          	lwip_write(sd, buf, len);
          	lwip_write(sd,str, strlen(str));
        }
       if(is_cmd_pushb(req+6))
        {
        	generate_http_header(buf, "txt", 1);
        	lwip_write(sd, buf, len);
        	lwip_write(sd,str2, strlen(str2));
           	strncpy(str2,"\0",20);
        }
	return 0;
}
void toggle_task()
{
	while(1)
	{
		if(taskrun==1 && ledon==0)
		{
			*(volatile unsigned int*)LEDS_TOGGLE=0x0;
			break;
		}
		else if(taskrun==1 && ledon==1)
		{
			break;
		}

			*(volatile unsigned int*)LEDS_TOGGLE=0x03;
			vTaskDelay(80);//while(j<5000)j++;j=0;
			*(volatile unsigned int*)LEDS_TOGGLE=0x0c;
			vTaskDelay(100);//while(j<5000)j++;j=0;
			*(volatile unsigned int*)LEDS_TOGGLE=0x30;
			vTaskDelay(80);//while(j<5000)j++;j=0;
			*(volatile unsigned int*)LEDS_TOGGLE=0xc0;
			vTaskDelay(100);//while(j<5000)j++;j=0;
			*(volatile unsigned int*)LEDS_TOGGLE=0x30;
			vTaskDelay(80);//while(j<5000)j++;j=0;
			*(volatile unsigned int*)LEDS_TOGGLE=0x0c;
			vTaskDelay(100);//while(j<5000)j++;j=0;
			*(volatile unsigned int*)LEDS_TOGGLE=0x03;
			vTaskDelay(80);//while(j<5000)j++;j=0;
	}

	if(ledon==1 && taskrun==1)
	*(volatile unsigned int*)LEDS_TOGGLE=0xFF;
	vTaskDelete(xToggletask);
		vTaskDelete(NULL);
}
/* respond for a file GET request */
int do_http_get(int sd, char *req, int rlen)
{
	//for continous polling for push button status..when webserver starts.
	xTaskCreate(push_button_thread, "push_buttob", 2048, (void *)sd, 2 , NULL);
	char filename[MAX_FILENAME];
	char buf[4096];
	int fsize, hlen, n;
	int fd;
	char *fext;


	/* determine file name */
	extract_file_name(filename, req, rlen, MAX_FILENAME);
	void mfs_lock();
	/* respond with 404 if not present */
	if (mfs_exists_file(filename) == 0) {
		//xil_printf("requested file %s not found, returning 404\r\n", filename);
		do_404(sd, req, rlen);
		return -1;
	}
	void mfs_unlock();
	/* respond with correct file */

	/* debug statement on UART */
        xil_printf("http GET: %s\r\n", filename);

    /* get a pointer to file extension */
	fext = get_file_extension(filename);
	void mfs_lock();
	fd = mfs_file_open(filename, MFS_MODE_READ);

	/* obtain file size,
	 * note that lseek with offset 0, MFS_SEEK_END does not move file pointer */
	fsize = mfs_file_lseek(fd, 0, MFS_SEEK_END);
	void mfs_unlock();
	/* write the http headers */
	hlen = generate_http_header(buf, fext, fsize);
	if (lwip_write(sd, buf, hlen) != hlen) {
		//xil_printf("error writing http header to socket\r\n");
		//xil_printf("http header = %s\r\n", buf);
		return -1;
	}

	/* now write the file */
	while (fsize) {
		int w;
		void mfs_lock();
		n = mfs_file_read(fd, buf, BUFSIZE);
		void mfs_unlock();

		if ((w = lwip_write(sd, buf, n)) != n) {
			//xil_printf("error writing file (%s) to socket, remaining unwritten bytes = %d\r\n",
					//filename, fsize - n);
			//xil_printf("attempted to lwip_write %d bytes, actual bytes written = %d\r\n", n, w);
			break;
		}

		fsize -= n;
	}
	void mfs_lock();
	mfs_file_close(fd);
	void mfs_unlock();

	return 0;
}

enum http_req_type { HTTP_GET, HTTP_POST, HTTP_UNKNOWN };
enum http_req_type decode_http_request(char *req, int l)
{
	char *get_str = "GET";
	char *post_str = "POST";

	if (!strncmp(req, get_str, strlen(get_str)))
		return HTTP_GET;

	if (!strncmp(req, post_str, strlen(post_str)))
		return HTTP_POST;

	return HTTP_UNKNOWN;
}

/* generate and write out an appropriate response for the http request */
int generate_response(int sd, char *http_req, int http_req_len)
{
//	xil_printf("INSIDE GENERATE RESPONSE FUNCTION\n");
	enum http_req_type request_type = decode_http_request(http_req, http_req_len);

	switch(request_type) {
	case HTTP_GET:
		return do_http_get(sd, http_req, http_req_len);
	case HTTP_POST:
		return do_http_post(sd, http_req, http_req_len);
	default:
		return do_404(sd, http_req, http_req_len);
	}
}
